@@ -91,7 +91,7 @@ ExtensionPluginInfo animExtensionPluginInfo = {
     .nEffects		= NUM_EFFECTS,
     .effects		= animEffects,
 
-    .nEffectOptions	= ANIMADDON_SCREEN_OPTION_NUM,
+    .nEffectOptions	= ANIMADDON_SCREEN_OPTION_NUM - NUM_NONEFFECT_OPTIONS,
 
     .prePaintOutputFunc	= polygonsPrePaintOutput
 };
@@ -573,6 +573,8 @@ static Bool animInitWindow(CompPlugin * p, CompWindow * w)
 	return FALSE;
 
     aw->eng.polygonSet = NULL;
+    aw->eng.numPs = 0;
+    aw->eng.ps = NULL;
 
     w->base.privates[as->windowPrivateIndex].ptr = aw;
 
@@ -585,6 +587,25 @@ static void animFiniWindow(CompPlugin * p, CompWindow * w)
 {
     ANIMADDON_WINDOW (w);
 
+    // We need to interrupt and clean up the animation currently being played
+    // by animationaddon for this window (if any)
+    if (aw->com->animRemainingTime > 0 &&
+	(aw->com->curAnimEffect == AnimEffectAirplane ||
+	 aw->com->curAnimEffect == AnimEffectBeamUp ||
+	 aw->com->curAnimEffect == AnimEffectBurn ||
+	 aw->com->curAnimEffect == AnimEffectDomino ||
+	 aw->com->curAnimEffect == AnimEffectExplode ||
+	 aw->com->curAnimEffect == AnimEffectFold ||
+	 aw->com->curAnimEffect == AnimEffectGlide3 ||
+	 aw->com->curAnimEffect == AnimEffectLeafSpread ||
+	 aw->com->curAnimEffect == AnimEffectRazr ||
+	 aw->com->curAnimEffect == AnimEffectSkewer))
+    {
+	ANIMADDON_DISPLAY (w->screen->display);
+
+	ad->animBaseFunctions->postAnimationCleanup (w);
+    }
+
     free(aw);
 }
 
@@ -627,8 +648,9 @@ animGetObjectOptions (CompPlugin *plugin,
 	(GetPluginObjectOptionsProc) animGetScreenOptions
     };
 
+    *count = 0;
     RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
-		     (void *) (*count = 0), (plugin, object, count));
+		     NULL, (plugin, object, count));
 }
 
 static CompBool
