@@ -283,11 +283,12 @@ typedef struct _GroupTabBar {
     GroupAnimationType bgAnimation;
 
     PaintState state;
-    int        timeoutHandle;
     int        animationTime;
     Region     region;
     int        oldWidth;
 
+    CompTimeoutHandle timeoutHandle;
+
     /* For DnD animations */
     int   leftSpringX, rightSpringX;
     int   leftSpeed, rightSpeed;
@@ -341,7 +342,8 @@ struct _GroupSelection {
     ChangeTabAnimationDirection nextDirection;
     GroupTabBarSlot             *nextTopTab;
 
-    GroupTabBarSlot *activateTab;
+    /* check focus stealing prevention after changing tabs */
+    Bool checkFocusAfterTabChange;
 
     GroupTabBar *tabBar;
 
@@ -397,12 +399,13 @@ typedef struct _GroupDisplay {
     Atom groupWinPropertyAtom;
     Atom resizeNotifyAtom;
 
-    Bool textAvailable;
+    TextFunc *textFunc;
 } GroupDisplay;
 
 /*
  * GroupScreen structure
  */
+
 typedef struct _GroupScreen {
     int windowPrivateIndex;
 
@@ -419,11 +422,12 @@ typedef struct _GroupScreen {
     WindowUngrabNotifyProc        windowUngrabNotify;
     DamageWindowRectProc          damageWindowRect;
     WindowStateChangeNotifyProc   windowStateChangeNotify;
+    ActivateWindowProc            activateWindow;
 
     GroupPendingMoves   *pendingMoves;
     GroupPendingGrabs   *pendingGrabs;
     GroupPendingUngrabs *pendingUngrabs;
-    Bool                dequeueTimerSet;
+    CompTimeoutHandle   dequeueTimeoutHandle;
 
     GroupSelection *groups;
     GroupSelection tmpSel;
@@ -448,6 +452,8 @@ typedef struct _GroupScreen {
     Bool              dragged;
     int               prevX, prevY; /* Buffer for mouse coordinates */
 
+    CompTimeoutHandle initialActionsTimeoutHandle;
+
     CompTexture glowTexture;
 } GroupScreen;
 
@@ -612,6 +618,8 @@ groupDamageWindowRect (CompWindow *w,
 		       Bool       initial,
 		       BoxPtr     rect);
 
+void
+groupActivateWindow (CompWindow *w);
 
 /*
  * cairo.c
@@ -810,7 +818,6 @@ groupResizeTabBarRegion (GroupSelection *group,
 void
 groupDamageTabBarRegion (GroupSelection *group);
 
-
 /*
  * paint.c
  */
